#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    input_Nx = 256;
    input_Ny = 256;
    UDP_packet_size = 16384;
    output_Nx = 512;
    output_Ny = 512;
    connected_to_server = false;

    // image
    displayed_image = QImage (output_Nx, output_Ny, QImage::Format_Grayscale8);
    for (int i=0 ; i<output_Nx ; i++)
    {
        for (int j=0 ; j<output_Ny ; j++)
        {
            displayed_image.setPixel(i,j,qRgb((i+j)%256,(i+j)%256,(i+j)%256));
        }
    }
    graph = new QGraphicsScene (this);
    graph->addPixmap(QPixmap::fromImage(displayed_image));
    ui->graphicsView_image_display->setScene(graph);

    input_image = QImage (input_Nx, input_Ny, QImage::Format_Grayscale8);

    QObject::connect(this, SIGNAL(image_ready()), this, SLOT(print_image()));

    sinlen = sizeof(SOCKADDR_IN);
    stream = false;
    data = new uint8_t[UDP_packet_size];
}

MainWindow::~MainWindow()
{
    delete ui;
    delete data;
}

void MainWindow::on_pushButton_edit_nx_ny_clicked()
{
    int nnx = ui->lineEdit_nx->text().toInt();
    int nny = ui->lineEdit_ny->text().toInt();

    for (int i=0 ; i<output_Nx ; i++)
    {
        for (int j=0 ; j<output_Ny ; j++)
        {
            displayed_image.setPixel(i,j,qRgb(255,255,255));
        }
    }
    graph->addPixmap(QPixmap::fromImage(displayed_image));
    ui->graphicsView_image_display->setScene(graph);

    output_Nx = nnx;
    output_Ny = nny;
    //delete(data);
    //data = new uint8_t[UDP_pakcet_size];

    displayed_image.scaled(nnx, nny);
    if (!connected_to_server)
    {
        std::cout << "lala, nnx = " << output_Nx << "nny = " << output_Ny << std::endl;
        for (int i=0 ; i<output_Nx ; i++)
        {
            for (int j=0 ; j<output_Ny ; j++)
            {
                displayed_image.setPixel(i,j,qRgb((i+j)%256,(i+j)%256,(i+j)%256));
            }
        }
        graph->addPixmap(QPixmap::fromImage(displayed_image));
        ui->graphicsView_image_display->setScene(graph);
    }
}

void MainWindow::image_thread()
{
    std::cout << "begin image thread" << std::endl;
    char trigger[1];
    trigger[0] = 'a';
    int l=0;
    int packet_len = 0;
    int packet_ID = 0;
    int packet_size = 16384;
    sendto(client_socket, trigger, 1, MSG_WAITALL, (SOCKADDR *)(&client_sin), sinlen);
    while (stream)
    {
        std::cout << "begin stream loop" << std::endl;
        packet_len = recvfrom(client_socket, (void*)data, packet_size, 0, (SOCKADDR *)(&client_sin), &sinlen);
        std::cout << "packet length: " << packet_len << std::endl;
        if(packet_len == 1)
        {
            packet_ID= int(data[0]);
            std::cout << "packet ID: " << packet_ID << std::endl;
            packet_len = recvfrom(client_socket, (void *)data, packet_size, 0, (SOCKADDR *)(&client_sin), &sinlen);
            std::cout << "second packet length: " << packet_len << std::endl;
        }
        if (packet_len == packet_size)
        {
            for (int k=0 ; k<packet_size ; k++)
            {
                l = 16384*packet_ID+k;
                input_image.setPixel(l/input_Ny,l%input_Ny,qRgb(data[k], data[k], data[k]));
            }
        }

        /*for (int i=0 ; i<Nx ; i++)
        {
            for (int j=0 ; j<Ny ; j++)
            {
                image.setPixel(i,j,qRgb(data[i*Nx+j], data[i*Nx+j], data[i*Nx+j]));
            }
        }*/
        if (packet_ID == 3)
        {
            emit image_ready();
        }
    }
}

void MainWindow::print_image()
{
    for (int i=0 ; i<input_Nx ; i++)
    {
        for (int j=0 ; j<input_Ny ; j++)
        {
            displayed_image.setPixel(2*i,2*j,input_image.pixel(i,j));
            displayed_image.setPixel(2*i,2*j+1,input_image.pixel(i,j));
            displayed_image.setPixel(2*i+1,2*j,input_image.pixel(i,j));
            displayed_image.setPixel(2*i+1,2*j+1,input_image.pixel(i,j));
        }
    }
    graph->addPixmap(QPixmap::fromImage(displayed_image));
    ui->graphicsView_image_display->setScene(graph);
}

void MainWindow::on_pushButton_edit_server_clicked()
{
    const char *IP = ui->lineEdit_IP->text().toStdString().c_str();
    init_UDP_client2(&client_socket, &client_sin, IP, 7610);
    ui->pushButton_start_image->setEnabled(true);
}

void MainWindow::on_pushButton_start_image_clicked()
{
    stream = true;
    image_construction_thread = std::thread([this](){image_thread();});
    image_construction_thread.detach();
    ui->pushButton_start_image->setEnabled(false);
    ui->pushButton_stop_image->setEnabled(true);
}


void MainWindow::on_pushButton_stop_image_clicked()
{
    ui->pushButton_stop_image->setEnabled(false);
    ui->pushButton_start_image->setEnabled(true);
    stream = false;
    usleep(100000);

}
